with Ada.Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Fourier_Init;
use Ada.Text_IO, Rep_Ops, Error_Ops, Numeric_Ops,Fourier_Init;
with Fou2_Ops;

generic

  Size_1, Size_2, Q_deg, P_deg, QPdeg: in Positive;
  type Scalar is private;

  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function IsInterval(S: Scalar) return Boolean is <>;

  with procedure Get(F: in File_Type; S: out Scalar) is <>;
  with procedure Put(F: in File_Type; S: in Scalar) is <>;
  with procedure Show(S: in Scalar) is <>;

  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Scal(E: Error) return Scalar is <>;

  with function Center0(S: Scalar) return Boolean is <>;
  with function Err0(S: Scalar) return Boolean is <>;
  with function Center(S: Scalar) return Rep is <>;
  with function ErrPart(S: Scalar) return Error is <>;
  with procedure ResetCenter(S: in out Scalar) is <>;
  with function ResetCenter(S: Scalar) return Scalar is <>;
  with procedure Split(S: in Scalar; Sr,Se: out Scalar) is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function Err(S: Scalar) return Error is <>;
  with procedure ResetErr(S: in out Scalar) is <>;
  with function ResetErr(S: Scalar) return Scalar is <>;
  with procedure ErrMult(Factor: in Error; S: in out Scalar) is <>;

  with function "abs"(S: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with function "+"(Left,Right: Scalar) return Scalar is <>;
  with function "-"(Left,Right: Scalar) return Scalar is <>;
  with function "*"(S: Scalar; E: Error) return Error is <>;
  with function "*"(E: Error; S: Scalar) return Error is <>;
  with function "*"(Left,Right: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(E: Error; S: Scalar) return Error is <>;
  with function "/"(Left,Right: Scalar) return Scalar is <>;
  with function "**"(X1: Scalar; I: Integer) return Scalar is <>;

  with function Sqrt(S: Scalar) return Scalar is <>;
  with function ExpAbsLog(S: Scalar) return Scalar is <>;
  with function Cosh(S: Scalar) return Scalar is <>;
  with function Sinh(S: Scalar) return Scalar is <>;
  with procedure CoshSinh(S: in Scalar; Sc,Ss: out Scalar) is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with function Golden return Scalar is <>;




package Combined_Fou2_Ops is

   package Scalar_Part is
      new Fou2_Ops (Size_1 => Size_1, Size_2 => Size_2, Q_Deg => Q_Deg,
                    P_Deg  => P_Deg,  QPdeg =>  QPdeg, Scalar => Scalar);
   package Numeric_Part is
      new Fou2_Ops (Size_1 => Size_1, Size_2 => Size_2, Q_Deg => Q_Deg,
                    P_Deg  => P_Deg,  QPdeg =>  QPdeg, Scalar => Numeric);


   package Scalar_Part1 renames Scalar_Part.Fou1_Ops;
   package Numeric_Part1 renames Numeric_Part.Fou1_Ops;
   subtype S_Fourier2 is Scalar_Part.Fourier2;
   subtype N_Fourier2 is Numeric_Part.Fourier2;
   procedure PutNumeric(Hs: in S_Fourier2; Hn: out N_Fourier2);              -- Approximate scalar Hs by numeric Hn
   function PutNumeric(Hs: S_Fourier2) return N_Fourier2;                    -- Return numeric approx of Hs
   procedure GetNumeric(Hn: in N_Fourier2; Hs: out S_Fourier2);              -- Convert numeric Hn to scalar Hs
   function GetNumeric(Hn: N_Fourier2) return S_Fourier2;                    -- Return scalar version of Hn
   procedure Inv(Err: in Radius; R: in Weights; H1: in S_Fourier2;
                                                H2: out S_Fourier2);                                       -- H2 := 1/H1
   function Inv(Err: Radius; R: Weights; H: S_Fourier2) return S_Fourier2;   -- Inv := 1/H
   procedure FixComp3(Err: in Radius; Rp3: in Radius; H: in S_Fourier2;
                                                      Hz: out S_Fourier2);                                  -- Hz Solves -H(.,.,.+Hz)=Hz

   end Combined_Fou2_Ops;





