with Ada.Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Fourier_Init;
use Ada.Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Fourier_Init;
with Fourier_Ops;

generic

  Size_1, Size_2, Q_deg, P_deg, QPdeg: in Positive;
  type Scalar is private;

  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function IsInterval(S: Scalar) return Boolean is <>;

  with procedure Get(F: in File_Type; S: out Scalar) is <>;
  with procedure Put(F: in File_Type; S: in Scalar) is <>;
  with procedure Show(S: in Scalar) is <>;

  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Scal(E: Error) return Scalar is <>;

  with function Center0(S: Scalar) return Boolean is <>;
  with function Err0(S: Scalar) return Boolean is <>;
  with function Center(S: Scalar) return Rep is <>;
  with function ErrPart(S: Scalar) return Error is <>;
  with procedure ResetCenter(S: in out Scalar) is <>;
  with function ResetCenter(S: Scalar) return Scalar is <>;
  with procedure Split(S: in Scalar; Sr,Se: out Scalar) is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function Err(S: Scalar) return Error is <>;
  with procedure ResetErr(S: in out Scalar) is <>;
  with function ResetErr(S: Scalar) return Scalar is <>;
  with procedure ErrMult(Factor: in Error; S: in out Scalar) is <>;

  with function "abs"(S: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with function "+"(Left,Right: Scalar) return Scalar is <>;
  with function "-"(Left,Right: Scalar) return Scalar is <>;
  with function "*"(S: Scalar; E: Error) return Error is <>;
  with function "*"(E: Error; S: Scalar) return Error is <>;
  with function "*"(Left,Right: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(E: Error; S: Scalar) return Error is <>;
  with function "/"(Left,Right: Scalar) return Scalar is <>;
  with function "**"(X1: Scalar; I: Integer) return Scalar is <>;

  with function Sqrt(S: Scalar) return Scalar is <>;
  with function ExpAbsLog(S: Scalar) return Scalar is <>;
  with function Cosh(S: Scalar) return Scalar is <>;
  with function Sinh(S: Scalar) return Scalar is <>;
  with procedure CoshSinh(S: in Scalar; Sc,Ss: out Scalar) is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with function Golden return Scalar is <>;

package Combined_Fourier_Ops is

  package Scalar_Part is
    new Fourier_Ops (Size_1 => Size_1, Size_2 => Size_2, Q_Deg => Q_Deg,
                     P_Deg  => P_Deg,  QPdeg =>  QPdeg, Scalar => Scalar);
  package Numeric_Part is
    new Fourier_Ops (Size_1 => Size_1, Size_2 => Size_2, Q_Deg => Q_Deg,
                     P_Deg  => P_Deg,  QPdeg =>  QPdeg, Scalar => Numeric);

  subtype S_Fourier is Scalar_Part.Fourier;
  subtype N_Fourier is Numeric_Part.Fourier;

  procedure PutNumeric(Hs: in S_Fourier; Hn: out N_Fourier);              -- Approximate scalar Hs by numeric Hn
  function PutNumeric(Hs: S_Fourier) return N_Fourier;                    -- Return numeric approx of Hs
  procedure GetNumeric(Hn: in N_Fourier; Hs: out S_Fourier);              -- Convert numeric Hn to scalar Hs
  function GetNumeric(Hn: N_Fourier) return S_Fourier;                    -- Return scalar version of Hn

  procedure Inv(Err: in Radius; R: in Weights; H1: in S_Fourier;
                H2: out S_Fourier);                                       -- H2 := 1/H1
  function Inv(Err: Radius; R: Weights; H: S_Fourier) return S_Fourier;   -- Inv := 1/H
  procedure FixComp3(Err: in Radius; Rp3: in Radius; H: in S_Fourier;
                     Hz: out S_Fourier);                                  -- Hz Solves -H(.,.,.+Hz)=Hz

end Combined_Fourier_Ops;

