with Ada.Text_IO, Rep_Ops, Error_Ops, Taylor_Init, Fourier_Init;
use Ada.Text_IO, Rep_Ops, Error_Ops, Taylor_Init, Fourier_Init;
with Fourier_Ops;


generic

   Size_1, Size_2, Q_deg, P_deg, QPdeg: in Positive;
   type Scalar is private;

   with function IsNumeric(S: Scalar) return Boolean is <>;
   with function IsInterval(S: Scalar) return Boolean is <>;
   with procedure Get(F: in File_Type; S: out Scalar) is <>;
   with procedure Put(F: in File_Type; S: in Scalar) is <>;
   with procedure Show(S: in Scalar) is <>;
   with function Scal(K: Integer) return Scalar is <>;
   with function Scal(R: Rep) return Scalar is <>;
   with function Scal(E: Error) return Scalar is <>;
   with function Center0(S: Scalar) return Boolean is <>;
   with function Err0(S: Scalar) return Boolean is <>;
   with function Center(S: Scalar) return Rep is <>;
   with function ErrPart(S: Scalar) return Error is <>;
   with procedure ResetCenter(S: in out Scalar) is <>;
   with function ResetCenter(S: Scalar) return Scalar is <>;
   with procedure Split(S: in Scalar; Sr,Se: out Scalar) is <>;
   with function Inf(S: Scalar) return Rep is <>;
   with function Sup(S: Scalar) return Rep is <>;
   with function SupAbs(S: Scalar) return Rep is <>;
   with function Err(S: Scalar) return Error is <>;
   with procedure ResetErr(S: in out Scalar) is <>;
   with function ResetErr(S: Scalar) return Scalar is <>;
   with procedure ErrMult(Factor: in Error; S: in out Scalar) is <>;
   with function "abs"(S: Scalar) return Scalar is <>;
   with function "-"(S: Scalar) return Scalar is <>;
   with function "+"(Left,Right: Scalar) return Scalar is <>;
   with function "-"(Left,Right: Scalar) return Scalar is <>;
   with function "*"(S: Scalar; E: Error) return Error is <>;
   with function "*"(E: Error; S: Scalar) return Error is <>;
   with function "*"(Left,Right: Scalar) return Scalar is <>;
   with function Inv(S: Scalar) return Scalar is <>;
   with function "/"(E: Error; S: Scalar) return Error is <>;
   with function "/"(Left,Right: Scalar) return Scalar is <>;
   with function "**"(X1: Scalar; I: Integer) return Scalar is <>;
   with function Sqrt(S: Scalar) return Scalar is <>;
   with function ExpAbsLog(S: Scalar) return Scalar is <>;
   with function Cosh(S: Scalar) return Scalar is <>;
   with function Sinh(S: Scalar) return Scalar is <>;
   with procedure CoshSinh(S: in Scalar; Sc,Ss: out Scalar) is <>;
   with function Exp(S: Scalar) return Scalar is <>;
   with function Log(S: Scalar) return Scalar is <>;
   with function Golden return Scalar is <>;

package Fou2_Ops is

   package Fou1_Ops is new Fourier_Ops (Size_1 => Size_1, Size_2 => Size_2, Q_Deg => Q_Deg,P_Deg  => P_Deg,  QPdeg =>  QPdeg, Scalar => Scalar);

   package Tay1_Ops renames Fou1_Ops.Component_Ops;
   use Fou1_Ops, Tay1_Ops;

   XE: Integer:=0;
   type Fourier2 is private;


   -- Below, H_J denotes the part of H of parity J

   procedure CopySign(H1: in Fourier2; H2: in out Fourier2);                         -- Copy signs of H1-components to H2
   procedure Put(F: in File_Type; R: in Weights; H: in Fourier2);                    -- Put H into F
   procedure Get(F: in File_Type; R: in Weights; H: out Fourier2);                   -- Get H from F
   procedure GetEven(F: in File_Type; R: in Weights; H: out Fourier2);               -- Get even H F
   procedure Read(Name: in String; R: in Weights; H: out Fourier2);                  -- Read H from file Name
   procedure ReadEven(Name: in String; R: in Weights; H: out Fourier2);              -- Read even H from file Name
   procedure SetRho(R: in Weights; H: in out Fourier2);                              -- H.R:=R
   function IMin(I1,I2: Integer) return Integer renames Integer'Min;
   function IMax(I1,I2: Integer) return Integer renames Integer'Max;
   procedure AddComponent(J: in Parity; M: in Freq1; N: in Freq2;
                                                     P: in Taylor;
                                                     H: in out Fourier2);            -- H_J[M,N] := H_J[M,N]+P
   procedure AddComponent(M: in Freq1; N: in Freq2; Pc: in Taylor;
                                                    Ps: in Taylor;
                                                    H: in out Fourier2);             -- H_0[M,N]:=H_0[M,N]+Pc, H_1[M,N]:=H_1[M,N]+Ps
   procedure AddComponent(J: in Parity; M: in Freq1; N: in Freq2;
                                                     K: in Degree;
                                                     S: in Scalar;
                                                     H: in out Fourier2);            -- H_J[M,N,K] := H_J[M,N,K]+S
   procedure AddComponent(M: in Freq1; N: in Freq2; K: in Degree;
                                                    S1,S2: in Scalar;
                                                    H: in out Fourier2);             -- H_0[M,N,K]:=H_0[M,N,K]+S1, H_1[M,N,K]:=H_1[M,N,K]+Ps
   function AddComponent(J: Parity; M: Freq1; N: Freq2;
                                              K: Degree;
                                              S: Scalar;
                                              H: Fourier2) return Fourier2;          -- Returns H_J[M,N,K]+S
   function AddComponent(M: Freq1; N: Freq2; K: Degree; S1: Scalar;
                                                        S2: Scalar;
                                                        H: Fourier2) return Fourier2;-- Returns H_0[M,N,K]+S1, H_1[M,N,K]:=H_1[M,N,K]+Ps
   procedure PiTranslateY(H1: in Fourier2; H2: out Fourier2);                        -- H2(x,y,z)=H1(x,y+Pi,z)
   procedure PiTranslateX(H1: in Fourier2; H2: out Fourier2);                        -- H2(x,y,z)=H1(x+Pi,y,z)
   procedure Write(F: in File_Type; H: in Fourier2);                                 -- Write H into F
   procedure Small_Gig(L: in Cutoffs; H1: in Fourier2; H2: out Fourier2);            -- H2 := ((omega*Grad)/(omega*Grad))H1
   procedure Gig(L: in Cutoffs; H1: in Fourier2; H2: out Fourier2);                  -- H2 := ((Omega*Grad)/(omega*Grad))H1
   procedure Gig(L: in Cutoffs; H: in out Fourier2);                                 -- H := ((Omega*Grad)/(omega*Grad))H
   procedure Ig(L: in Cutoffs; H1: in Fourier2; H2: out Fourier2);                   -- H2 := (1/(omega*Grad))H1
   procedure Ig(L: in Cutoffs; H: in out Fourier2);                                  -- H := (1/(omega*Grad))H
   procedure Dig(L: in Cutoffs; H1: in Fourier2; H2: out Fourier2);                  -- H2 := (D_3/(omega*Grad))H1
   procedure Dig(L: in Cutoffs; H: in out Fourier2);                                 -- H := (D_3/(omega*Grad))H
   function Norm(H: Fourier2) return Error;                                          -- Norm := ||H||_R  where R=Rho(H)
   function SupNorm(H: Fourier2) return Error;                                       -- Norm := sup_(M,N,K,J) {||H_J[M,N,K]||_R} where R=Rho(H)
   function Norm(R: Weights; H: Fourier2) return Error;                              -- Norm := ||H||_R
   function SupNorm(R: Weights; H: Fourier2) return Error;                           -- Norm := sup_(K) {||H_J[M,N,K]||_R}
   function ShortNorm(R: Weights; H: Fourier2) return Error;                         -- Norm :=||H without the unstable direction||_R
   function Norm(F: Scalar_Array; R3: Radius; H: Fourier2) return Error;             -- Norm := F-weighted ||H||_R3
   function Deg(J: Parity; M: Freq1; N: Freq2; H: Fourier2) return Integer;          -- Returns Deg of H_J[M,N]
   function EffDeg(J: Parity; M,N: integer; H: Fourier2) return Integer;             -- Returns Effective Deg of H_J[M,N]
   function EffDeg(M,N: integer; H: Fourier2) return Integer;                        -- Returns Effective Deg of H[M,N]
   procedure EffDeg(J: in Parity; M,N: in Integer; H: in Fourier2; Dk: out Integer); -- Effective Deg of H_J[M,N]
   procedure EffDeg(M,N: in Integer; H: in Fourier2; Dk: out Integer);               -- Effective Deg of H[M,N]
   procedure EffDeg(J: in Parity; M: in Integer; H: in Fourier2; Dn,Dk: out Integer);-- Effective Degs of H_J[M]
   procedure EffDeg(M: in Integer; H: in Fourier2; Dn,Dk: out Integer);              -- Effective Degs of H[M]
   procedure EffDeg(J: in Parity; H: in Fourier2; Dm,Dn,Dk: out Integer);            -- Effective Degs of H_J
   procedure EffDeg(H: in Fourier2; Dm,Dn,Dk: out Integer);                          -- Effective Degs of H
   procedure SetZero(R: in Weights; H: out Fourier2);                                -- H := 0
   procedure ZeroAverage(H: in out Fourier2);                                        -- H_{0,0} := 0
   procedure Copy(H1: in Fourier2; H2: out Fourier2);                                -- H2 := H1
   procedure NProj(H1: in Fourier2; H2: out Fourier2);                               -- Projects H_0 on even N's, H_1 on odd N's
   procedure NProj(H1: in out Fourier2);                                             -- Projects H_0 on even N's, H_1 on odd N's
   procedure MProj(H1: in out Fourier2);                                             -- Projects H_0 on even M's, H_1 on odd M's
   procedure MNProj(H1: in out Fourier2);                                            -- Projects H_0 on even N's and M's, H_1 on odd N's and M's
   procedure Eliminate(H1: in Fourier2; H2: out Fourier2);                           -- Eliminates even M's and N's from H_0, od from H_1
   procedure Neg(H1: in Fourier2; H2: out Fourier2);                                 -- H2 := -H1
   function "-"(H: Fourier2) return Fourier2;                                        -- Returns -H
   procedure Mult(S: in Scalar; H: in out Fourier2);                                 -- H := S*H
   procedure ShortMult(S: in Scalar; H: in out Fourier2);                            -- H := S*H without the unstable direction
   procedure ShortMult(S: in Scalar; H1: in Fourier2; H2: out Fourier2);             -- H2 := S*H1 without the unstable direction
   procedure Prod(S: in Scalar; H1: in Fourier2; H2: out Fourier2);                  -- H2 := S*H1
   procedure Prod(H1: in Monom; H2: in Fourier2; H3: out Fourier2;
                                                 D: in Degrees := Dmax);             -- H3 := H1*H2
   procedure Prod(H1,H2: in Fourier2; H3: out Fourier2;D: in Degrees := Dmax);       -- H3 := H1*H2   (I^minus)
   procedure Prod(R: in Weights; H1,H2: in Fourier2; H3: out Fourier2;
                                                     D: in Degrees := Dmax);         -- H3:=H1*H2
   function "*"(S: Scalar; H: Fourier2) return Fourier2;                             -- Returns S*H
   procedure Add(H1: in Fourier2; H2: in out Fourier2);                              -- H2 := H2+H1
   procedure Sum(H1,H2: in Fourier2; H3: out Fourier2);                              -- H3 := H1+H2
   function "+"(H1,H2: Fourier2) return Fourier2;                                    -- Returns H1+H2
   procedure Sub(H1: in Fourier2; H2: in out Fourier2);                              -- H2 := H2-H1
   procedure Diff(H1,H2: in Fourier2; H3: out Fourier2);                             -- H3 := H1-H2
   function "-"(H1,H2: Fourier2) return Fourier2;                                    -- Returns H1-H2
   procedure MultAdd(S: in Scalar; H1: in Fourier2; H2: in out Fourier2);            -- H2 := H2+S*H1
   procedure MultAdd(R: in Weights; P: in Taylor; H1: in Fourier2;
                                                  H2: in out Fourier2;
                                                  D: in Degrees := Dmax);            -- H2 := H2+P*H1
   procedure MultAdd(J: in Parity; S: in Scalar; M,N: in Integer;
                                                 P: in Taylor;
                                                 H2: in Fourier2;
                                                 H3: in out Fourier2;
                                                 D: in Degrees := Dmax);             -- H3 := H3+S*Monom(M,N,P)*H2
   procedure MultAdd(H1: in Monom; H2: in Fourier2; H3: in out Fourier2;
                                                    D: in Degrees := Dmax);          -- H3:=H3+H1*H2
   procedure Iminus(L: in Cutoffs; H1: in Fourier2; H2: out Fourier2;
                                                    Collect: in Boolean := True);    -- H2 := I^minus H1
   procedure Iminus(L: in Cutoffs; H: in out Fourier2);                              -- H := I^minus H
   function Iminus(L: Cutoffs; H: Fourier2) return Fourier2;                         -- Iminus := I^minus H
   procedure Iplus(L: in Cutoffs; H1: in Fourier2; H2: out Fourier2;
                                                   Collect: in Boolean := True);     -- H2 := I^plus H1
   procedure Iplus(L: in Cutoffs; H: in out Fourier2);                               -- H := I^plus H
   function Iplus(L: Cutoffs; H: Fourier2) return Fourier2;                          -- Iplus := I^minus H
   procedure ZcAdd(I: in Index3; H1: in Fourier2; H2: in out Fourier2);              -- H2[M,N,0] := H2[M,N,0]+H1[M,N,I] ...
   function "*"(H1,H2: Fourier2) return Fourier2;                                    -- Returns H1*H2
   procedure Scale(Z,S: in Scalar; H1: in Fourier2; H2: out Fourier2);               -- H2 := Z*H1(.,.,S.)
   function Scale(Z,S: Scalar; H: Fourier2) return Fourier2;                         -- Scale := Z*H(.,.,S.)
   procedure Der3(R3: in Radius; H1: in Fourier2; H2: out Fourier2);                 -- H2 := D_3 H1
   function Der3(R3: Radius; H: Fourier2) return Fourier2;                           -- Der3 := D_3 H
   procedure DerDer3(R3: in Radius; H1: in Fourier2; H2: out Fourier2);              -- H2 := D_3 D_3 H1
   function DerDer3(R3: Radius; H: Fourier2) return Fourier2;                        -- DerDer3 := D_3 D_3 H
   procedure NumInv(Err: in Radius; H1: in Fourier2; H2: out Fourier2);              -- H2 := 1/H1
   function  NumInv(Err: Radius; H1: Fourier2) return Fourier2;                      -- Returns 1/H1
   procedure CosSin(H: in Fourier2; Hc: out Fourier2; Hs: out Fourier2);             -- Hc:=Cos(H), Hs:=Sin(H)
   procedure CosSin_New(H: in Fourier2; Hc: out Fourier2; Hs: out Fourier2);         -- Hc:=Cos(H), Hs:=Sin(H), another version
   function Component(J: Parity; M,N: Integer; H: Fourier2) return Taylor;           -- Returns H_J[M,N]
   function Component(J: Parity; M,N: Integer; K: Natural;
                                               H: Fourier2) return Scalar;           -- Returns H_J[M,N,K]
   function NumVal(X,Y,Z: Scalar; H: Fourier2) return Scalar;                        -- Returns H(X,Y,Z)
   function Max(S1: Integer; S2: Integer) return Integer;
   function Rho(H: Fourier2) return Weights;
   function Comp3(H,H3: Fourier2) return Fourier2;                                   -- Returns H(.,.,H3)
   procedure Comp3(H,H3: in Fourier2; Hr: out Fourier2);                             -- Hr := H(.,.,H3)
   procedure SetComponent(J: in Parity; M: in Freq1; N: in Freq2;
                                                     P: in Taylor;
                                                     H: in out Fourier2);            -- H_J[M,N] := P
   procedure SetComponent(J: in Parity; M: in Freq1; N: in Freq2;
                                                     K: in Degree;
                                                     S: in Scalar;
                                                     H: in out Fourier2);            -- H_J[M,N,K] := S
   function Coeff(J: Parity; M,N: Integer; H: Fourier2) return Taylor;               -- Coeff := H_J_{M,N}
   function Coeff(J: Parity; M,N: Integer; K: Natural; H: Fourier2) return Scalar;   -- Coeff := H_J_{M,N,K}
   procedure NumFixComp3(Err: in Radius; H: in Fourier2; Hz: out Fourier2);          -- Hz solves -H(.,.,.+Hz)=Hz
   procedure NumInvComp3(Err: in Radius; H1,H2: in Fourier2; H3: out Fourier2);      -- H3 solves H2(.,.,H3)=H1
   function  NumFixComp3(Err: Radius; H: Fourier2) return Fourier2;                  -- Return solution Hz of -H(.,.,.+Hz)=Hz
   procedure NumDqDot(R: in Weights; S1,S2: in Scalar; H1: in Fourier2;
                                                       H2: out Fourier2);            -- H2 := (S1*D_1+S2*D_2) H1
   function NumDqDot(R: in Weights; S1,S2: in Scalar;
                                    H1: in Fourier2) return Fourier2;                -- H2 := (S1*D_1+S2*D_2) H1
   procedure NumDer1(R1: in Radius; H1: in Fourier2; H2: out Fourier2);              -- H2 := D_1 H1
   procedure NumDer2(R2: in Radius; H1: in Fourier2; H2: out Fourier2);              -- H2 := D_2 H1
   procedure CosSinX(H: in Fourier2; Hc,Hs: out Fourier2);                           -- Hc := Cos(X+H), Hs := Sin(X+H)
   procedure CosSinY(H: in Fourier2; Hc,Hs: out Fourier2);                           -- Hc := Cos(Y+H), Hs := Sin(Y+H)
   procedure Split(H: in Fourier2; Hc,He: out Fourier2);                             -- Split H into polynomial plus error
   procedure CompOmega12(R: in Weights; H,Hq: in Fourier2; Hr: out Fourier2);        -- Hr := H(.+Hq*Omega,.)
   procedure CompOmega12(R: in Weights; H1,H2,Hq: in Fourier2;                       -- Hr1 := H1(.+Hq*Omega,.)
                                        Hr1,Hr2: out Fourier2);                      -- Hr2 := H2(.+Hq*Omega,.)
   procedure CompOmegaTmu0(R: in Weights; H,Hq: in Fourier2; Hr: out Fourier2);      -- Hr := H(.+T_mu_0 (Hq*Omega),.)
   procedure Rotation(R: in Weights; S1,S2: in Scalar; H,Hq: in Fourier2;
                                                       Hr: out Fourier2);            -- Hr := H(.+(S1,S2)*Hq,.)
   procedure Show(H: in Fourier2; Cut: in Rep := Zero);                              -- Showing all components of H larger then Cut in abs value
   procedure ShowLogNorms(H: in Fourier2);                                           -- Show norms of components H_{M,N}
   procedure Cshow(F: in File_Type; H: in Fourier2);                                 -- Output for DS_Tool (as a C code)
   procedure NumValQ0(H: in Fourier2; P: out Taylor);                                -- P := H(0,0,.)
   procedure PiOver2TranslateY(H1: in out Fourier2);                                 -- H2(x,y,z)=H1(x,y+Pi/2,z)
   procedure PiOver2TranslateX(H1: in out Fourier2);                                 -- H2(x,y,z)=H1(x+Pi/2,y,z)
   procedure MinusPiOver2TranslateX(H1: in out Fourier2);                            -- H2(x,y,z)=H1(x-Pi/2,y,z)

   ----- auxiliary procedures
   procedure CutDeg(R: in Weights; Cut: in Rep; H: in out Fourier2);                 -- Cut h.o. terms of Norm<Cut
   procedure NumInormalize(R: in Weights; H: in out Fourier2);                       -- Normalize H s.t. <H,H>_R=1
   function NumIprod(R: Weights; H1,H2: Fourier2) return Scalar;                     -- NumIprod := <H1,H2>_R

   ----useful for testing
   procedure Random(Seed: in out Long_Integer; R: in Weights;
                                               H: out Fourier2; L: in Cutoffs);      -- H := (random function)
   procedure AddRandom(Seed: in out Long_Integer; J: in Parity; M: in Freq1;
                                                                N: in Freq2;
                                                                K: in Degree;
                                                                H: in out Fourier2); -- H_J[M,N,K]:=H_J[M,N,K]+(random number)
   procedure RandomOld(Seed: in out Long_Integer; R: in Weights;
                                                  H: out Fourier2);                  -- H := (random function), old version
   procedure NextRandom(I: in out Long_Integer);                                     -- randomly resetting Integer

private
   type Fourier2 is
      record
         SIN: Fourier;
         COS: Fourier;
         R: Weights;
      end record;
end Fou2_Ops;








