with Ada.Text_IO, Rep_Ops, Error_Ops, Fourier_Init;
use Ada.Text_IO, Rep_Ops, Error_Ops, Fourier_Init;
with Taylor_Ops;

generic

   Size_1, Size_2, Q_deg, P_deg, QPdeg: in Positive;
   type Scalar is private;

   with function IsNumeric(S: Scalar) return Boolean is <>;
   with function IsInterval(S: Scalar) return Boolean is <>;
   with procedure Get(F: in File_Type; S: out Scalar) is <>;
   with procedure Put(F: in File_Type; S: in Scalar) is <>;
   with procedure Show(S: in Scalar) is <>;
   with function Scal(K: Integer) return Scalar is <>;
   with function Scal(R: Rep) return Scalar is <>;
   with function Scal(E: Error) return Scalar is <>;
   with function Center0(S: Scalar) return Boolean is <>;
   with function Err0(S: Scalar) return Boolean is <>;
   with function Center(S: Scalar) return Rep is <>;
   with function ErrPart(S: Scalar) return Error is <>;
   with procedure ResetCenter(S: in out Scalar) is <>;
   with function ResetCenter(S: Scalar) return Scalar is <>;
   with procedure Split(S: in Scalar; Sr,Se: out Scalar) is <>;
   with function Inf(S: Scalar) return Rep is <>;
   with function Sup(S: Scalar) return Rep is <>;
   with function SupAbs(S: Scalar) return Rep is <>;
   with function Err(S: Scalar) return Error is <>;
   with procedure ResetErr(S: in out Scalar) is <>;
   with function ResetErr(S: Scalar) return Scalar is <>;
   with procedure ErrMult(Factor: in Error; S: in out Scalar) is <>;
   with function "abs"(S: Scalar) return Scalar is <>;
   with function "-"(S: Scalar) return Scalar is <>;
   with function "+"(Left,Right: Scalar) return Scalar is <>;
   with function "-"(Left,Right: Scalar) return Scalar is <>;
   with function "*"(S: Scalar; E: Error) return Error is <>;
   with function "*"(E: Error; S: Scalar) return Error is <>;
   with function "*"(Left,Right: Scalar) return Scalar is <>;
   with function Inv(S: Scalar) return Scalar is <>;
   with function "/"(E: Error; S: Scalar) return Error is <>;
   with function "/"(Left,Right: Scalar) return Scalar is <>;
   with function "**"(X1: Scalar; I: Integer) return Scalar is <>;
   with function Sqrt(S: Scalar) return Scalar is <>;
   with function ExpAbsLog(S: Scalar) return Scalar is <>;
   with function Cosh(S: Scalar) return Scalar is <>;
   with function Sinh(S: Scalar) return Scalar is <>;
   with procedure CoshSinh(S: in Scalar; Sc,Ss: out Scalar) is <>;
   with function Exp(S: Scalar) return Scalar is <>;
   with function Log(S: Scalar) return Scalar is <>;
   with function Golden return Scalar is <>;

package Fourier_Ops is

   pragma Elaborate_Body;

   Pdeg: constant Positive := Positive'Min(P_Deg,QPdeg);
   Qdeg: constant Positive := Positive'Min(Q_Deg,QPdeg);
   Size1: constant Positive := Positive'Min(Size_1,Qdeg);
   Size2: constant Positive := Positive'Min(Size_2,Qdeg);

   package Component_Ops is new Taylor_Ops (Size => Pdeg, Scalar => Scalar);
   use Component_Ops;

   subtype Freq1 is Integer range -Size1 .. Size1;
   subtype Freq2 is Integer range 0 .. Size2;
   subtype Index3 is Integer range 0 .. Pdeg;
   type Degrees is array (Freq1,Freq2) of Degree;
   type Scalar_Array is array (Freq1,Freq2) of Scalar;

   type Fourier is private;
   type Monom is private;

   Dmax: Degrees;               ---- in old programs: Dp
   DmaxCount: Natural;          ---- in old programs: Lmax

   procedure CopySign(H1: in Fourier; H2: in out Fourier);                       -- Copy the sign of H1-components to H2
   function Nmin(M: in Integer; J: in Parity) return Integer;                    -- Nmin := smallest_allowed_N
   function Nmax(M: in Integer) return Integer;                                  -- Nmax := largest_allowed_N
   function InRange(M: in Integer; N: in Integer;
                                   J: in Parity) return Boolean;                 -- InRange := (M and N are in range)
   function Deg(M: Freq1; N: Freq2; H: Fourier) return Integer;                  -- Deg of H[M,N]
   function EffDeg(M,N: integer; H: Fourier) return Integer;                     -- Effective Deg of H[M,N]
   procedure EffDeg(M,N: in Integer; H: in Fourier; Dk: out Integer);            -- Effective Deg of H[M,N]
   procedure EffDeg(M: in Integer; H: in Fourier; Dn,Dk: out Integer);           -- Effective Degs of H[M]
   procedure EffDeg(H: in Fourier; Dm,Dn,Dk: out Integer);                       -- Effective Degs of H
   procedure ShowR(R: in Weights);                                               -- Show R
   procedure ShowJR(H: in Fourier);                                              -- Show parity and weights
   procedure CheckPar(H: in Fourier;
                      M: in Integer :=0; N: in Integer :=0);                     -- Parity consistency check
   function Par(H: Fourier) return Parity;                                       -- Par := Parity of H  (0=even 1=odd)
   procedure SetPar(J: in Parity; H: in out Fourier);                            -- Set Parity of H     (0=even 1=odd)
   function Rho(H: Fourier) return Weights;                                      -- Rho := weights defining domain of H
   procedure SetRho(R: in Weights; H: in out Fourier);                           -- Set Weights defining domain of H
   function Min(R1,R2: Weights) return Weights;                                  -- Min of R1 and R2

   procedure Component(M,N: in Integer; H: in Fourier; P: out Taylor);           -- P := H[M,N]
   function Component(M,N: Integer; H: Fourier) return Taylor;                   -- Component := H[M,N]
   function Component(M,N: Integer; K: Natural; H: Fourier) return Scalar;       -- Component := H[M,N,K]
   procedure SetComponent(M: in Freq1; N: in Freq2; P: in Taylor;
                                                    H: in out Fourier);          -- H[M,N] := P
   procedure SetComponentNoCheck(M: in Freq1; N: in Freq2; P: in Taylor;
                                                       H: in out Fourier);       --without the parity check
   procedure SetComponentNoCheck(M: in Freq1; N: in Freq2; K: in Degree;
                                                       S: in Scalar;
                                                       H: in out Fourier);       --without the parity check
   procedure AddComponent(M: in Freq1; N: in Freq2; P: in Taylor;
                                                    H: in out Fourier);          -- H[M,N] := H[M,N]+P
   procedure MultComponent(M: in Freq1; N: in Freq2; S: in Scalar;
                                                     H: in out Fourier);         -- H[M,N] := S*H[M,N]
   procedure SetComponent(M: in Freq1; N: in Freq2; K: in Degree;
                                                    S: in Scalar;
                                                    H: in out Fourier);          -- H[M,N,K] := S
   procedure AddComponent(M: in Freq1; N: in Freq2; K: in Degree;
                                                    S: in Scalar;
                                                    H: in out Fourier);          -- H[M,N,K] := H[M,N,K]+S
   procedure AddComponentNoCheck(M: in Freq1; N: in Freq2; K: in Degree;
                                                           S: in Scalar;
                                                           H: in out Fourier);   --without the parity check
   procedure PiTranslateY(H1: in Fourier; H2:  out Fourier);                     -- H2(x,y,z)=H1(x,y+Pi,z)
   procedure PiTranslateX(H1: in Fourier; H2:  out Fourier);                     -- H2(x,y,z)=H1(x+Pi,y,z)
   procedure ShortPut(I: in Integer);                                            -- Short current output of Integer
   procedure ShortPut(F: in File_Type; I: in Integer);                           -- Short output of Integer into F
   function NormFactor(R: Weights; M,N: Integer) return Scalar;                  -- Norm of function E_{M,N}
   procedure NormFactors(R: in Weights; F: out Scalar_array);                    -- Norms of functions E_{M,N}
   function Coeff(M,N: Integer; H: Fourier) return Taylor;                       -- Coeff := H_{M,N}
   function Coeff(M,N: Integer; K: Natural; H: Fourier) return Scalar;           -- Coeff := H_{M,N,K}
   procedure SetZero(J: in Parity; R: in Weights; H: out Fourier);               -- H := 0
   procedure ZeroAverage(H: in out Fourier);                                     -- H_{0,0} := 0
   procedure Split(H: in Fourier; Hc,He: out Fourier);                           -- Split H into polynomial plus error
   function Center0(H: Fourier) return Boolean;                                  -- True iff polynomial part is zero
   function Err0(H: Fourier) return Boolean;                                     -- True iff error part is zero
   procedure ErrConvert(H: in Fourier; He: out Fourier);                         -- Enlarge H -> He centered at 0
   function ErrConvert(H: Fourier) return Fourier;                               -- Enlarge H -> ErrConvert centered at 0
   procedure Get(F: in File_Type; J: in Parity; R: in Weights; H: out Fourier);  -- Read H from F
   procedure Put(F: in File_Type; J: in Parity; H: in Fourier);                  -- Write H to F
   procedure Show(H: in Fourier; Cut: in Rep := Zero);                           -- Show coeffs of H  (of modulus >= Cut)
   procedure Cshow(F: in File_Type; J: in Parity; H: in Fourier);                -- Output for DS_Tool (as a C code)
   procedure Show(H1,H2: in Fourier; Cut: in Rep := Zero);                       -- Show coeffs of H1 and H2
   procedure Read(Name: in String; J: in Parity; R: in Weights; H: out Fourier); -- Read H from file Name
   procedure Write(Name: in String; J: in Parity :=0; H: in Fourier);            -- Write H to file Name
   function Norm(R: Weights; H: Fourier) return Error;                           -- Norm := ||H||_R
   function Norm(F: Scalar_Array; R3: Radius; H: Fourier) return Error;          -- Norm := F-weighted ||H||_R  where R=Rho(H)
   function Norm(H: Fourier) return Error ;                                      -- Norm := ||H||_R  where R=Rho(H)
   function SupNorm(R: Weights; H: Fourier) return Error;                        -- Norm := ||H||_R
   function SupNorm(F: Scalar_Array; R3: Radius; H: Fourier) return Error;       -- Norm := F-weighted sup_(K) {||H[M,N,K]||_R3}
   function SupNorm(H: Fourier) return Error ;                                   -- Norm := sup_(M,N,K) {||H[M,N,K]||_R} where R=Rho(H)
   function MaxAbsComponent(H: Fourier) return Rep;                              -- Return largest |H[M,N,K]|
   procedure ShowNorms(H: in Fourier; Cut: in Rep := Zero);                      -- Show H with norms
   procedure ShowLogNorms(H: in Fourier);                                        -- Show norms of components H_{M,N}
   procedure Copy(H1: in Fourier; H2: out Fourier);                              -- H2 := H1
   procedure OddEliminate(H1: in Fourier; H2: out Fourier);                      -- Projection on even K's
   procedure EvenEliminate(H1: in Fourier; H2: out Fourier);                     -- Projection on odd  K's
   procedure OddNProj(H1: in Fourier; H2: out Fourier);                          -- Projection on odd N's
   procedure OddMProj(H1: in Fourier; H2: out Fourier);                          -- Projection on odd M's
   procedure OddMNProj(H1: in Fourier; H2: out Fourier);                         -- Projection on odd M's and N's
   procedure EvenNProj(H1: in Fourier; H2: out Fourier);                         -- Projection on even N's
   procedure EvenMProj(H1: in Fourier; H2: out Fourier);                         -- Projection on even M's
   procedure EvenMNProj(H1: in Fourier; H2: out Fourier);                        -- Projection on even M's and N's
   procedure Neg(H1: in Fourier; H2: out Fourier);                               -- H2 := -H1
   function "-"(H: Fourier) return Fourier;                                      -- Return -H
   procedure Mult(S: in Scalar; H: in out Fourier);                              -- H := S*H
   procedure Prod(S: in Scalar; H1: in Fourier; H2: out Fourier);                -- Mult := S*H
   function "*"(S: Scalar; H: Fourier) return Fourier;                           -- Return S*H
   procedure Add(H1: in Fourier; H2: in out Fourier);                            -- H2 := H2+H1
   procedure Add(J: in Parity; H1: in Fourier; H2: in out Fourier);              -- H2 := H2+H1
   procedure Sum(H1,H2: in Fourier; H3: out Fourier);                            -- H3 := H1+H2
   function "+"(H1,H2: Fourier) return Fourier;                                  -- Return H1+H2
   procedure Sub(H1: in Fourier; H2: in out Fourier);                            -- H2 := H2-H1
   procedure Diff(H1,H2: in Fourier; H3: out Fourier);                           -- H3 := H1-H2
   function "-"(H1,H2: Fourier) return Fourier;                                  -- Return H1-H2
   procedure MultAdd(S: in Scalar; H1: in Fourier; H2: in out Fourier);          -- H2 := H2+S*H1
   procedure MultAdd(R: in Weights; P: in Taylor; H1: in Fourier;
                                                  H2: in out Fourier;
                                                  D: in Degrees := Dmax);        -- H2 := H2+P*H1
   procedure MultAdd(S: in Scalar; M,N: in Integer; P1: in Taylor;
                                                    H2: in Fourier;
                                                    H3: in out Fourier;
                                                    D: in Degrees := Dmax);      -- H3 := H3+S*P*H2
   procedure ZcAdd(I: in Index3; H1: in Fourier; H2: in out Fourier);            -- H2[M,N,0] := H2[M,N,0]+H1[M,N,I] ...
   procedure CollectErr(H: in out Fourier);                                      -- Collect errors in M=N=0 term
   procedure Iminus(L: in Cutoffs; H1: in Fourier; H2: out Fourier;
                                                   Collect: in Boolean := True); -- H2 := I^minus H1
   procedure Iminus(L: in Cutoffs; H: in out Fourier);                           -- H := I^minus H
   function Iminus(L: Cutoffs; H: Fourier) return Fourier;                       -- Iminus := I^minus H
   procedure Iplus(L: in Cutoffs; H1: in Fourier; H2: out Fourier;
                                                  Collect: in Boolean := True);  -- H2 := I^plus H1
   procedure Iplus(L: in Cutoffs; H: in out Fourier);                            -- H := I^plus H
   function Iplus(L: Cutoffs; H: Fourier) return Fourier;                        -- Iplus := I^minus H
   procedure Normalize(M,N: in Integer; M3,N3: out Integer;
                                        Flip,Simple: out Boolean);               -- normalize and check frequency
   function Mon(J: Parity; M: Freq1; N: Freq2; K: Index3;
                                               C: Scalar) return Monom;          -- Return monomial with ...
   procedure Prod(H1: in Monom; H2: in Fourier; H3: out Fourier;
                                                D: in Degrees := Dmax);          -- H3 := H1*H2
   procedure MultAdd(H1: in Monom; H2: in Fourier; H3: in out Fourier;
                                                   D: in Degrees := Dmax);       -- H3 := H3+H1*H2
   procedure Prod(R: in Weights; H1,H2: in Fourier; H3: out Fourier;
                                                    D: in Degrees := Dmax);      -- H3 := H1*H2   (I^minus)
   function "*"(H1,H2: Fourier) return Fourier;                                  -- Return H1*H2
   procedure Scale(Z,S: in Scalar; H1: in Fourier; H2: out Fourier);             -- H2 := Z*H1(.,.,S.)
   function Scale(Z,S: Scalar; H: Fourier) return Fourier;                       -- Scale := Z*H(.,.,S.)
   procedure Der3(R3: in Radius; H1: in Fourier; H2: out Fourier);               -- H2 := D_3 H1
   function Der3(R3: Radius; H: Fourier) return Fourier;                         -- Der3 := D_3 H
   procedure DerDer3(R3: in Radius; H1: in Fourier; H2: out Fourier);            -- H2 := D_3 D_3 H1
   function DerDer3(R3: Radius; H: Fourier) return Fourier;                      -- DerDer3 := D_3 D_3 H
   procedure Comp(R3: in Radius; P: in Taylor; H1: in Fourier;
                                               H2: out Fourier);                 -- H2 := P o H1
   function Comp(R3: Radius; P: Taylor; H: Fourier) return Fourier;              -- Comp := P o H1
   procedure Comp3(H,H3: in Fourier; Hr: out Fourier);                           -- Hr := H(.,.,H3)
   function Comp3(H,H3: Fourier) return Fourier;                                 -- Comp3 := H(.,.,H3)
   procedure CosSin(H: in Fourier; Hc,Hs: out Fourier);                          -- Hc := Cos(H), Hs := Sin(H)
   procedure CosSinX(H: in Fourier; Hc,Hs: out Fourier);                         -- Hc := Cos(X+H), Hs := Sin(X+H)
   procedure CosSinY(H: in Fourier; Hc,Hs: out Fourier);                         -- Hc := Cos(Y+H), Hs := Sin(Y+H)
   procedure CompOmega12(R: in Weights; H,Hq: in Fourier;
                                        Hr: out Fourier);                        -- Hr := H(.+Hq*Omega,.)

   ----- procedures that are implemented only numerically
   procedure CompOmega12(R: in Weights; H1,H2,Hq: in Fourier;                    -- Hr1 := H1(.+Hq*Omega,.)
                                        Hr1,Hr2: out Fourier);                   -- Hr2 := H2(.+Hq*Omega,.)
   procedure NumValQ0(H: in Fourier; P: out Taylor);                             -- P := H(0,0,.)
   function NumVal(X,Y,Z: Scalar; H: Fourier) return Scalar;                     -- NumVal := H(X,Y,Z)
   procedure NumInv(Err: in Radius; H1: in Fourier; H2: out Fourier);            -- H2 := 1/H1
   procedure NumInvComp3(Err: in Radius; H1,H2: in Fourier;
                                         H3: out Fourier);                       -- H3 solves H2(.,.,H3)=H1
   procedure NumFixComp3(Err: in Radius; H: in Fourier;
                                         Hz: out Fourier);                       -- Hz solves -H(.,.,.+Hz)=Hz
   function NumFixComp3(Err: Radius; H: Fourier) return Fourier;                 -- Return solution Hz of -H(.,.,.+Hz)=Hz
   procedure NumDer1(R1: in Radius; H1: in Fourier; H2: out Fourier);            -- H2 := D_1 H1
   procedure NumDer2(R2: in Radius; H1: in Fourier; H2: out Fourier);            -- H2 := D_2 H1
   procedure NumDqDot(R: in Weights; S1,S2: in Scalar;
                                     H1: in Fourier; H2: out Fourier);           -- H2 := (S1*D_1+S2*D_2) H1
   procedure Gig(L: in Cutoffs; H: in out Fourier);                              -- H := ((Omega*Grad)/(omega*Grad))H
   procedure Gig(L: in Cutoffs; H1: in Fourier; H2: out Fourier);                -- H2 := ((Omega*Grad)/(omega*Grad))H1
   procedure Ig(L: in Cutoffs; H1: in Fourier; H2: out Fourier);                 -- H2 := (1/(omega*Grad))H1
   procedure Small_Gig(L: in Cutoffs; H1: in Fourier; H2: out Fourier);          -- H2 := ((omega*Grad)/(omega*Grad))H1
   procedure Dig(L: in Cutoffs; H: in out Fourier);                              -- H := (D_3/(omega*Grad))H
   procedure Dig(L: in Cutoffs; H1: in Fourier; H2: out Fourier);                -- H2 := (D_3/(omega*Grad))H1
   procedure Digg(L: in Cutoffs; H: in out Fourier);                             -- H := (D_3/(omega*Grad))H
   procedure Digg(L: in Cutoffs; H1: in Fourier; H2: out Fourier);               -- H2 := (D_3/(omega*Grad))H1
   function  GetParity(H: Monom) return Integer;                                 -- returns the parity of H
   function  GetParity(H: Fourier) return Integer;                               -- returns the parity of H


   ----- auxiliary procedures

   procedure MinDeg(H1: in Fourier; H2: in out Fourier);                         -- Make degrees of H2 <= degrees of H1
   procedure CutDeg(R: in Weights; Cut: in Rep; H: in out Fourier);              -- Cut h.o. terms of Norm<Cut
   function NumIprod(R: Weights; H1,H2: Fourier) return Scalar;                  -- NumIprod := <H1,H2>_R
   procedure NumInormalize(R: in Weights; H: in out Fourier);                    -- Normalize H s.t. <H,H>_R=1

   ----- useful for testing
   procedure Random(Seed: in out Long_Integer; J: in Parity;
                                               R: in Weights; H: out Fourier);   -- H := (random function)
   procedure RandomOdd(Seed: in out Long_Integer; J: in Parity;
                                                  R: in Weights; H: out Fourier;
                                                                 L: in Cutoffs); -- H := (random function, odd under (q,p)->(-q,-p))


private

   type Taylor_Array is array (Freq1,Freq2) of Taylor;

   type Norm_Factors is
      record
         R: Weights;
         C: Scalar_Array;
      end record;

   type Fourier is
      record
         J: Parity;
         R: Weights;
         C: Taylor_Array;
      end record;

   type Monom is
      record
         J: Parity;
      M: Freq1;
      N: Freq2;
      K: Index3;
      C: Scalar;
    end record;

  SavedFactors: Norm_Factors;

  procedure MaxDegs(Degs: out Degrees; Count: out Natural);
  procedure UpdateSavedFactors(R: in Weights);
  procedure ShowIndices(J: in Parity; M: in Freq1; N: in Freq2; K: in Index3);
  procedure Decompose(M,N: in Integer; M1,N1,M2,N2: out Integer);               -- decompose into lower frequencies
  function LowFreq(M,N: Integer) return Boolean;                                -- check if (M,N) is small2

end Fourier_Ops;




