with Messages;
use Messages;

package body Interval_Ops is

   Epsilon: constant Error := Err(Two**(-52));
   Interval_Zero: constant Interval := (Zero,Zero_Error);
   Interval_One: constant Interval := (One,Zero_Error);
   Interval_Two: constant Interval := (Two,Zero_Error);


   function IsNumeric(S: Interval) return Boolean is
   begin
      return False;
   end IsNumeric;


   function IsInterval(S: Interval) return Boolean is
   begin
      return True;
   end IsInterval;


   procedure Get(F: in File_Type; S: out Interval) is
      R: Rep;
   begin
      Get(F,R);
      S := Scal(R);
   end Get;


   procedure Put(F: in File_Type; S: in Interval) is
   begin
      Put(F,S.R);
   end Put;


   procedure Show(S: in Interval) is
   begin
      Put(Current_Output,S.R);
      if S.E /= Zero_Error then
         Put(" [");
         Put(Current_Output,Sup(S.E),3,4);
         Put(" ]");
      end if;
      New_Line;
   end Show;


   function Scal(K: Integer) return Interval is
   begin
      return (Rep(K),Zero_Error);
   end Scal;


   function Scal(R: Rep) return Interval is
   begin
      return (R,Zero_Error);
   end Scal;


   function Scal(E: Error) return Interval is
   begin
      return (Zero,E);
   end Scal;


   function Scal(Rlow,Rhigh: Rep) return Interval is
      Rs,Rd: Rep;
   begin
      if Rlow<Rhigh then
         Rs := Half*(Rhigh+Rlow);
         Rd := Half*(Rhigh-Rlow);
         return (Rs,Err(Rd)+Rs*Epsilon+Rd*Epsilon);
      elsif Rlow>Rhigh then
         Message("Interval_Ops.Scal",Domain_Violation);
      end if;
      return (Rlow,Zero_Error);
   end Scal;


   function Center0(S: Interval) return Boolean is
   begin
      return S.R=Zero;
   end Center0;


   function Err0(S: Interval) return Boolean is
   begin
      return S.E=Zero_Error;
   end Err0;


   function Center(S: Interval) return Rep is
   begin
      return S.R;
   end;


   function ErrPart(S: Interval) return Error is
   begin
      return S.E;
   end ErrPart;


   procedure ResetCenter(S: in out Interval) is
   begin
      S.R := Zero;
   end ResetCenter;


   function ResetCenter(S: Interval) return Interval is
   begin
      return (Zero,S.E);
   end ResetCenter;


   procedure Split(S: in Interval; Sr,Se: out Interval) is
   begin
      Sr := (S.R,Zero_Error);
      Se := (Zero,S.E);
   end Split;


   function Inf(S: Interval) return Rep is
   begin
      return S.R-Sup(S.E);
   end Inf;


   function Sup(S: Interval) return Rep is
   begin
      return S.R+Sup(S.E);
   end Sup;


   function SupAbs(S: Interval) return Rep is
   begin
      return Abs(S.R)+Sup(S.E);
   end SupAbs;


   function Err(S: Interval) return Error is
   begin
      return Err(Abs(S.R))+S.E;
   end Err;


   procedure ResetErr(S: in out Interval) is
   begin
      S.E := Zero_Error;
   end ResetErr;


   function ResetErr(S: Interval) return Interval is
   begin
      return (S.R,Zero_Error);
   end ResetErr;


   procedure ErrMult(Factor: in Error; S: in out Interval) is
   begin
      S.E := Factor*S.E;
   end ErrMult;


   function "abs"(S: Interval) return Interval is
   begin
      if S.R >= Sup(S.E) then
         return S;
      elsif -S.R >= Sup(S.E) then
         return (-S.R,S.E);
      else
         Message("Interval_Ops.Abs",Not_Implemented);
         return Interval_Zero;
      end if;
   end "abs";


   function "-"(S: Interval) return Interval is
   begin
      return (-S.R,S.E);
   end "-";


   function "+"(Left,Right: Interval) return Interval is
      R: Rep;
   begin
      R := Left.R+Right.R;
      return (R,R*Epsilon+Left.E+Right.E);
   end "+";


   function "-"(Left,Right: Interval) return Interval is
      R: Rep;
   begin
      R := Left.R-Right.R;
      return (R,R*Epsilon+Left.E+Right.E);
   end "-";


   function "*"(S: Interval; E: Error) return Error is
   begin
      return (Abs(S.R)+Sup(S.E))*E;
   end "*";


   function "*"(E: Error; S: Interval) return Error is
   begin
      return (Abs(S.R)+Sup(S.E))*E;
   end "*";


   function "*"(Left,Right: Interval) return Interval is
      R: Rep;
   begin
      R := Left.R*Right.R;
      return (R,R*Epsilon+Left.R*Right.E+Right.R*Left.E+Left.E*Right.E);
   end "*";


   function Inv(S: Interval) return Interval is
      R0,R1,R2: Rep;
   begin
      R1 := Pred(Half/Sup(S));
      R2 := Succ(Half/Inf(S));
      if R1<Zero or else R2>Zero then
         R0 := R2+R1;
         R1 := Succ(R2-R1);
      else
         Message("Interval_Ops.Inv",Division_By_Zero);
      end if;
      return (R0,Err(R1)+R0*Epsilon);
   end Inv;


   function "/"(E: Error; S: Interval) return Error is
   begin
      return E*Inv(S);
   end "/";


   function "/"(Left,Right: Interval) return Interval is
   begin
      return Left*Inv(Right);
   end "/";


   function "**"(X1: Interval; I: Integer) return Interval is
      J: Integer;
      X,X2: Interval;
   begin
      if I=0 then
         return Interval_One;
      else
         if I>0 then
            J := I;
            X := X1;
         else
            J := -I;
            X := Inv(X1);
         end if;
         if (J rem 2)=0 then
            X2 := Interval_One;
         else
            X2 := X;
         end if;
         J := J/2;
         while J>0 loop
            X := X*X;
            if (J rem 2)>0 then
               X2 := X*X2;
            end if;
            J := J/2;
         end loop;
         return X2;
      end if;
   end "**";


   function Sqrt(S: Interval) return Interval is
      R,R1,R2,Eps: Rep;
   begin
      if S=Interval_Zero or else S=Interval_One then
         return S;
      end if;
      R := Sup(S);
      R2 := Sqrt(R);
      Eps := R2*Two**(-50);
      while R2*R2<=R loop
         R2 := R2+Eps;
      end loop;
      R := Inf(S);
      if R=Zero then
         return Scal(Zero,R2);
      end if;
      R1 := Sqrt(R);
      Eps := R1*Two**(-50);
      while R1*R1>=R loop
         R1 := R1-Eps;
      end loop;
      return Scal(R1,R2);
   end Sqrt;


   function ExpAbsLog(S: Interval) return Interval is
   begin
      if Inf(S)>=One then
         return S;
      elsif Sup(S)<=One then
         return Inv(S);
      else
         return Scal(One,Max(Sup(S),Sup(Inv(S))));
      end if;
   end ExpAbsLog;


   function Golden return Interval is
      S: Interval;
   begin
      S := (One+Half,Err(Half));
      for I in 1 .. 50 loop
         S := Interval_One+Inv(S);
      end loop;
      return S;
   end Golden;


   function "/"(S: Interval; R: Rep) return Interval is
      Rt: Rep;
   begin
      Rt := S.R/R;
      return(Rt,Rt*Epsilon+S.E/R);
   end "/";


   procedure CoshSinh(S: in Interval; Sc,Ss: out Interval) is
   begin
      if S = Interval_Zero then
         Sc := Interval_One;
         Ss := S;
      else
         declare
            K: Integer;
            R: Rep;
            X,T: Interval;
         begin
            K := 0;
            R := SupAbs(S);
            while R>Half loop
               K := K+1;
               R := R*Half;
            end loop;
            X := S*Scal(Half**K);
            T := X*X;
            Sc := Interval_One+Scal(Half*Unit_Error);
            Ss := Sc;
            for I in reverse 1 .. 6 loop
               Sc := Interval_One+Sc*T/Rep((2*I)*(2*I-1));
               Ss := Interval_One+Ss*T/Rep((2*I)*(2*I+1));
            end loop;
            Ss := X*Ss;
            for I in 1 .. K loop
               T := Sc;
               Sc := Interval_One+Interval_Two*Ss*Ss;
               Ss := Interval_Two*T*Ss;
            end loop;
         end;
      end if;
   end CoshSinh;


   function Cosh(S: Interval) return Interval is
      Sc,Ss: Interval;
   begin
      CoshSinh(S,Sc,Ss);
      return Sc;
   end Cosh;


   function Sinh(S: Interval) return Interval is
      Sc,Ss: Interval;
   begin
      CoshSinh(S,Sc,Ss);
      return Ss;
   end Sinh;


   function Exp(S: Interval) return Interval is
      Sc,Ss: Interval;
   begin
      CoshSinh(S,Sc,Ss);
      return Sc+Ss;
   end Exp;


   function Log(S: Interval) return Interval is
      Iter: constant Integer := 30;
      X,Y: Interval;
   begin
      if S=Interval_One then
         return Interval_Zero;
      end if;
      X := Interval_One-S;
      Y := Interval_One-Scal(SupAbs(X));
      if Inf(Y)>Zero then
         Y := Scal(Err(Inv(Y))/(Iter+1));
      else
         Message("Interval_Ops.Log",Domain_Violation);
      end if;
      for I in reverse 1 .. Iter loop
         Y := Inv(Scal(I))+X*Y;
      end loop;
      return -X*Y;
   end Log;

end Interval_Ops;
