with Ada.Text_IO, Rep_Ops, Error_Ops;
use Ada.Text_IO, Rep_Ops, Error_Ops;

package Interval_Ops is

   type Interval is private;

   function IsNumeric(S: Interval) return Boolean;
   function IsInterval(S: Interval) return Boolean;

   procedure Get(F: in File_Type; S: out Interval);          ---- center only
   procedure Put(F: in File_Type; S: in Interval);           ---- center only
   procedure Show(S: in Interval);

   function Scal(K: Integer) return Interval;
   function Scal(R: Rep) return Interval;
   function Scal(E: Error) return Interval;

   function Center0(S: Interval) return Boolean;
   function Err0(S: Interval) return Boolean;
   function Center(S: Interval) return Rep;
   function ErrPart(S: Interval) return Error;
   procedure ResetCenter(S: in out Interval);
   function ResetCenter(S: Interval) return Interval;
   procedure Split(S: in Interval; Sr,Se: out Interval);
   function Inf(S: Interval) return Rep;
   function Sup(S: Interval) return Rep;
   function SupAbs(S: Interval) return Rep;
   function Err(S: Interval) return Error;
   procedure ResetErr(S: in out Interval);
   function ResetErr(S: Interval) return Interval;
   procedure ErrMult(Factor: in Error; S: in out Interval);

   function "abs"(S: Interval) return Interval;
   function "-"(S: Interval) return Interval;
   function "+"(Left,Right: Interval) return Interval;
   function "-"(Left,Right: Interval) return Interval;
   function "*"(S: Interval; E: Error) return Error;
   function "*"(E: Error; S: Interval) return Error;
   function "*"(Left,Right: Interval) return Interval;
   function Inv(S: Interval) return Interval;
   function "/"(E: Error; S: Interval) return Error;
   function "/"(Left,Right: Interval) return Interval;
   function "**"(X1: Interval; I: Integer) return Interval;

   function Sqrt(S: Interval) return Interval;
   function ExpAbsLog(S: Interval) return Interval;
   function Golden return Interval;
   procedure CoshSinh(S: in Interval; Sc,Ss: out Interval);
   function Cosh(S: Interval) return Interval;
   function Sinh(S: Interval) return Interval;
   function Exp(S: Interval) return Interval;
   function Log(S: Interval) return Interval;

private

   type Interval is
      record
         R: Rep;
         E: Error;
      end record;

   function Scal(Rlow,Rhigh: Rep) return Interval;
   function "/"(S: Interval; R: Rep) return Interval;

end Interval_Ops;

