with Ada.Text_IO, Ada.Integer_Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Interval_Ops, Linear_Ops, Fourier_Init,Messages,Ada.Characters.Latin_1;   --*
use Ada.Text_IO, Ada.Integer_Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Interval_Ops, Linear_Ops, Fourier_Init,Messages, Ada.Characters.Latin_1;    --*
with RG2_Ops;

procedure Num_DRG is

  pragma Suppress( Storage_Check);

  subtype Scalar is Numeric;
  Zero: constant Scalar := Scal(0);

  MaxDegQ1: constant Positive := 7;
  MaxDegQ2: constant Positive := 7;
  MaxDegQ:  constant Positive := 7;
  MaxDegP:  constant Positive := 7;
  MaxDegQP: constant Positive := 9;
  Np:       constant Positive := 128;

  package Run_RG2_Ops is
     new RG2_Ops (Size_1 => MaxDegQ1, Size_2 => MaxDegQ2, Q_Deg => MaxDegQ,
                 P_Deg  => MaxDegP,   QPdeg =>  MaxDegQP , Scalar => Scalar);

  use Run_RG2_Ops;

  package Run_Num2_Ops renames Numeric_Fou2_Ops;
  package Run_Num1_Ops  renames Numeric_Fou1_Ops;
  package Run_Scalar_Taylor_Ops renames Scalar_Taylor_Ops;
  package Run_Numeric_Taylor_Ops renames Numeric_Taylor_Ops;
  package Run_Scalar2_Ops renames  Scalar_Fou2_Ops;
  package Run_Scalar1_Ops renames  Scalar_Fou1_Ops;
  use  Run_Num2_Ops, Run_Num1_Ops, Run_Scalar_Taylor_Ops, Run_Numeric_Taylor_Ops, Run_Scalar2_Ops, Run_Scalar1_Ops;

  Sigma: constant Rep := 1.10;
  Tau:   constant Rep := 1.50;
  Pw: constant Integer:=6;
  Mmin: constant Integer:=5;
  Mmax: constant Integer:=7;
  Eren: constant Radius := 1.0E-8;
  InvGolden: constant Scalar:=Golden-Scal(One);

  DirOut: constant String :="7_7_7_7_9/";
  DirInH: constant String :="7_7_7_7_9/";
  DirInPsi: constant String :="7_7_7_7_9/";
  SigTa: constant String:= "1.10_1.50/";
  PsiIname: constant String:=DirInPsi & SigTa & "Psi/psi_x";
  HIname: constant String:=DirInH & SigTa & "H/h_x";
  DRGOname: constant String:=DirInH & SigTa & "DRG/col";
  KK: constant String:= "_09";
  IK: constant Integer:=9;

  Kappa: Rep;
  PPi: ProjParam;
  Rfix,Rpri,Rpsi: Weights;
  H,Hr: Hamilton;
begin

   MaxI:=500;
   New_Line;
   Kappa := Sigma/Tau;
   MuMax:=Rep(InvGolden)*Rep(InvGolden);

   Rfix := (0.00, 0.85, 0.15);
   Rpri := (0.00, 0.85, 0.15);
   Rpsi := (0.00, 0.85, 0.15);

   Read(HIname & KK,Rfix,H);

   Put("# FIXED POINT:"); Put(Current_Output,IK);
   New_Line;
   Put("INPUT DIR:"); Put(DirInH);
   New_Line;
   Put("ERROR IN RENORMALIZATION:="); Put(Current_Output,Eren);
   New_Line;
   Put("Mmin:="); Put(Current_Output,Mmin); Put("   Mmax:="); Put(Current_Output,Mmax);
   New_Line;
   Put("Rfix:"); ShowRR(Rfix);
   Put("Rpri:"); ShowRR(Rpri);
   Put("Rpsi:"); ShowRR(Rpsi);
   New_Line;
   ShowParam(Sigma,Kappa);
   New_Line;
   New_Line;
   IminusDef(Sigma,Kappa,PPi);
   Put_Line("Iminus:");
   ShowProj(PPi);

   RG_Pw(PsiIname,PsiIname,Pw,IK,Eren,PPi,Rfix,Rpsi,H,Hr,True,False);

   New_Line;
   Put("NumDRG...");
   New_Line;
   NumDRG(PsiIname,DRGOname,Pw,IK,Mmin,Mmax,Eren,Rep(0.0),PPi,Rfix,Rpsi,H,Hr,True);
   New_Line;
   Put("Done");

end Num_DRG;
























































