with Messages;
use Messages;

package body Numeric_Ops is


   package Numeric_IO is new Ada.Text_IO.Float_IO(Numeric);


   function IsNumeric(S: Numeric) return Boolean is
   begin
      return True;
   end IsNumeric;


   function IsInterval(S: Numeric) return Boolean is
   begin
      return False;
   end IsInterval;


   procedure Get(F: in File_Type; S: out Numeric) is
   begin
      Numeric_IO.Get(File=>F, Item=>S);
   end Get;


   procedure Put(F: in File_Type; S: in Numeric) is
   begin
      Numeric_IO.Put(File=>F, Item=>S, Aft=>16, Exp=>4);
   end Put;


   procedure Show(S: in Numeric) is
   begin
      Put(Current_Output,S);
      New_Line;
   end Show;


   function Scal(K: Integer) return Numeric is
   begin
      return Numeric(K);
   end Scal;


   function Scal(R: Rep) return Numeric is
   begin
      return Numeric(R);
   end Scal;


   function Scal(E: Error) return Numeric is
   begin
      return Zero_Numeric;
   end Scal;


   procedure SetZero(S: in out Numeric) is
   begin
      S := Zero_Numeric;
   end SetZero;


   function ZeroFun(S: Numeric) return Numeric is
   begin
      return Zero_Numeric;
   end ZeroFun;


   function Copy(S: Numeric) return Numeric is
   begin
      return S;
   end Copy;


   function Center0(S: Numeric) return Boolean is
   begin
      return S=Zero_Numeric;
   end Center0;


   function Center(S: Numeric) return Rep is
   begin
      return Rep(S);
   end Center;


   function ErrPart(S: Numeric) return Error is
   begin
      return Zero_Error;
   end ErrPart;


   procedure Split(S: in Numeric; Sr,Se: out Numeric) is
   begin
      Sr := S;
      Se := Zero_Numeric;
   end Split;


   function Err(S: Numeric) return Error is
   begin
      return Err(Abs(Rep(S)));
   end Err;


   function SupAbs(S: Numeric) return Rep is
   begin
      return Abs(Rep(S));
   end SupAbs;


   procedure ResetErr(S: in out Numeric) is
   begin
      null;
   end ResetErr;


   procedure ErrMult(Factor: in Error; S: in out Numeric) is
   begin
      null;
   end ErrMult;


   function "*"(S: Numeric; E: Error) return Error is
   begin
      return Rep(S)*E;
   end "*";


   function "*"(E: Error; S: Numeric) return Error is
   begin
      return Rep(S)*E;
   end "*";


   function Inv(S: Numeric) return Numeric is
   begin
      return Numeric(One/Rep(S));
   end Inv;


   function "/"(E: Error; S: Numeric) return Error is
   begin
      return E/Rep(S);
   end "/";


   function Sqrt(S: Numeric) return Numeric is
   begin
      return Numeric(Sqrt(Rep(S)));
   end Sqrt;


   function ExpAbsLog(S: Numeric) return Numeric is
   begin
      if S >= Numeric(One) then
         return S;
      else
         if S <= Zero_Numeric then
            Message("Numeric_Ops.ExpAbsLog",Domain_Violation);
         end if;
         return Numeric(One/Rep(S));
      end if;
   end ExpAbsLog;


   function Cosh(S: Numeric) return Numeric is
   begin
      return Numeric(Cosh(Rep(S)));
   end Cosh;


   function Sinh(S: Numeric) return Numeric is
   begin
      return Numeric(Sinh(Rep(S)));
   end Sinh;


   procedure CoshSinh(S: in Numeric; Sc,Ss: out Numeric) is
   begin
      Sc := Numeric(Cosh(Rep(S)));
      Ss := Numeric(Sinh(Rep(S)));
   end CoshSinh;


   function Exp(S: Numeric) return Numeric is
   begin
      return Numeric(Exp(Rep(S)));
   end Exp;


   function Log(S: Numeric) return Numeric is
   begin
      return Numeric(Log(Rep(S)));
   end Log;


   function Golden return Numeric is
   begin
      return    Numeric((Sqrt(Five)+One)/Two);
   end Golden;

end Numeric_Ops;

