with Ada.Text_IO, Rep_Ops, Error_Ops;
use Ada.Text_IO, Rep_Ops, Error_Ops;

package Numeric_Ops is

   type Numeric is new Long_Float;

   Zero_Numeric: constant Numeric := Numeric(0);

   function IsNumeric(S: Numeric) return Boolean;
   function IsInterval(S: Numeric) return Boolean;

   procedure Get(F: in File_Type; S: out Numeric);
   procedure Put(F: in File_Type; S: in Numeric);
   procedure Show(S: in Numeric);

   function Scal(K: Integer) return Numeric;
   function Scal(R: Rep) return Numeric;
   function Scal(E: Error) return Numeric;

   procedure SetZero(S: in out Numeric);            --- to be renamed
   function ZeroFun(S: Numeric) return Numeric;     --- to be renamed
   function Copy(S: Numeric) return Numeric;        --- to be renamed

   function Center0(S: Numeric) return Boolean;
   function Err0(S: Numeric) return Boolean renames IsNumeric;
   function Center(S: Numeric) return Rep;
   function ErrPart(S: Numeric) return Error;
   procedure ResetCenter(S: in out Numeric) renames SetZero;
   function ResetCenter(S: Numeric) return Numeric renames ZeroFun;
   procedure Split(S: in Numeric; Sr,Se: out Numeric);
   function Inf(S: Numeric) return Rep renames Center;
   function Sup(S: Numeric) return Rep renames Center;
   function SupAbs(S: Numeric) return Rep;
   function Err(S: Numeric) return Error;
   procedure ResetErr(S: in out Numeric);
   function ResetErr(S: Numeric) return Numeric renames Copy;
   procedure ErrMult(Factor: in Error; S: in out Numeric);

   function "*"(S: Numeric; E: Error) return Error;
   function "*"(E: Error; S: Numeric) return Error;
   function Inv(S: Numeric) return Numeric;
   function "/"(E: Error; S: Numeric) return Error;

   function Sqrt(S: Numeric) return Numeric;
   function ExpAbsLog(S: Numeric) return Numeric;
   function Cosh(S: Numeric) return Numeric;
   function Sinh(S: Numeric) return Numeric;
   procedure CoshSinh(S: in Numeric; Sc,Ss: out Numeric);
   function Exp(S: Numeric) return Numeric;
   function Log(S: Numeric) return Numeric;
   function Golden return Numeric;

   --  pragma Inline(
   --    Scal,SetZero,ZeroFun,Copy,Center0,Center,ErrPart,SupAbs,Err,ErrMult,"*",Inv,"/",ExpAbsLog);

end Numeric_Ops;

