--A sample program that implements RG with contraction in two unstable directions;
-- the program looks for eigenvectors at each renormalization step
with Ada.Text_IO, Ada.Integer_Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Interval_Ops, Linear_Ops, Fourier_Init,Messages;   --*
use Ada.Text_IO, Ada.Integer_Text_IO, Rep_Ops, Error_Ops, Numeric_Ops, Interval_Ops, Linear_Ops, Fourier_Init,Messages;    --*
with RG2_Ops;

procedure RG is

   pragma Suppress( Storage_Check);

   subtype Scalar is Numeric;
   Zero: constant Scalar := Scal(0);

   MaxDegQ1: constant Positive := 7;
   MaxDegQ2: constant Positive := 7;
   MaxDegQ:  constant Positive := 7;
   MaxDegP:  constant Positive := 7;
   MaxDegQP: constant Positive := 9;
   Np:       constant Positive := 128;

   package Run_RG2_Ops is
      new RG2_Ops (Size_1 => MaxDegQ1, Size_2 => MaxDegQ2, Q_Deg => MaxDegQ,
                   P_Deg  => MaxDegP,   QPdeg =>  MaxDegQP , Scalar => Scalar);

   use Run_RG2_Ops,Run_RG2_Ops.Scalar_Fou2_Ops,Run_RG2_Ops.Scalar_Taylor_Ops, Run_RG2_Ops.Scalar_Fou1_Ops,Run_RG2_Ops.Numeric_Fou2_Ops,Run_RG2_Ops.Numeric_Fou1_Ops;

   Sigma: constant Rep := 1.10;
   Tau:   constant Rep := 1.50;
   IK: constant Integer:=9;
   Eren:  constant Radius := 1.0E-9;
   Pw: constant Integer:=6;
   InvGolden: constant Scalar:=Golden-Scal(One);

   DirOut: constant String :="7_7_7_7_9/";
   DirInPsi: constant String :="7_7_7_7_9/";
   DirInH: constant String :="7_7_7_7_9/";
   DirInEv: String :="7_7_7_7_9/";
   SigTa: constant String:= "1.10_1.50/";
   SigTaIn: constant String:= "1.10_1.50/";
   HOname: constant String:=DirOut & SigTa & "H/h_x";
   HIname: constant String:=DirInH & SigTa & "H/h_x";
   PsiIname: constant String:=DirInPsi & SigTa & "Psi/psi_x";
   PsiOname: constant String:=DirOut & SigTa & "Psi/psi_x";
   MuOname: constant String:=DirOut & SigTa & "Scaling_Data/mu";

   Kappa: Rep;
   PPi: ProjParam;
   Rfix,Rpri,Rpsi: Weights;
   F: File_Type;
   H,Hr,Hrr: Hamilton;
   Hpsi: Polynom;
   Mu: Mu_Array;
   KK:  String:="_09";
begin
   MaxI:=500;
   MuMax:=Rep(InvGolden)*Rep(InvGolden);

   Rfix := (0.0, 0.85, 0.15);      -- initial and final R for H
   Rpri := (0.0, 0.85, 0.15);      -- R for HoU  i.e. R'
   Rpsi := (0.0, 0.85, 0.15);      -- R for psi

   Put("INPUT H FILE:"); Put(HIname);
   New_Line;
   Put("OUTPUT H FILES:"); Put(HOname);
   New_Line;
   Put("INPUT PSI FILES:"); Put(PsiIname);Put(" and ");Put(PsiIname);
   New_Line;
   Put("OUTPUT PSI FILES:"); Put(PsiOname);Put(" and ");Put(PsiOname);
   New_Line;
   Put("# FIXED POINT (IN PERIOD 12):"); Put(Current_Output,IK);
   New_Line;
   Put("ERROR IN RENORMALIZATION:="); Put(Current_Output,Eren);
   New_Line;
   Put("Rfix:"); ShowRR(Rfix);
   Put("Rpri:"); ShowRR(Rpri);
   Put("Rpsi:"); ShowRR(Rpsi);

   Kappa := Sigma/Tau;

   New_Line;
   Put("PROJECTION PARAMETERS FOR H:");
   New_Line;
   ShowParam(Sigma,Kappa);
   New_Line;
   IminusDef(Sigma,Kappa,PPi);
   Put_Line("Iminus:");
   ShowProj(PPi);
   New_Line;

   Read(HIname & KK,Rfix,Hr);

   for K in IK..(IK+5) loop
      case (K mod 12) is
         when  0 => KK:="_00";
         when  1 => KK:="_01";
         when  2 => KK:="_02";
         when  3 => KK:="_03";
         when  4 => KK:="_04";
         when  5 => KK:="_05";
         when  6 => KK:="_06";
         when  7 => KK:="_07";
         when  8 => KK:="_08";
         when  9 => KK:="_09";
         when  10 => KK:="_10";
         when  11 => KK:="_11";
         when others =>  Message("RG",Parameter_Error);
      end case;

      --WRITING Hr
      if K/=IK then
         Create(F,Out_File,HOname & KK);
         Write(F,Hr);
         Close(F);
      end if;

      New_Line;
      Put("'MOST CRITICAL' HAMILTONIAN #" & KK);
      New_Line;
      Put("******************************************************************************************************");
      New_Line;
      Show(Hr);
      Put("******************************************************************************************************");
      New_Line;

      --RENORMALIZATION
      Read(PsiIname & KK,Rfix,Hpsi);

      RG2(Eren,PPi,Rpsi,Rfix,Hr,Hpsi,Hrr,0);
      Mu(K mod 12):=Rmu;

      Create(F,Out_File,PsiOname & KK);
      Write(F,Hpsi);
      Close(F);

      Hr:=Hrr;
   end loop;

   New_Line;
   Put("WRITING MU'S...");
   --WRITING MU'S
   Create(F,Out_File,MuOname);
   for K in IK..(IK+5) loop
      Put(F,Mu(K mod 12));
   end loop;
   Close(F);
   Put(" DONE");
end RG;
























































