with Ada.Text_IO, Ada.Strings.Bounded, Rep_Ops, Error_Ops, Numeric_Ops, Fourier_Init, Linear_Ops,Taylor_Ops;
use Ada.Text_IO, Ada.Strings.Bounded, Rep_Ops, Error_Ops, Numeric_Ops, Fourier_Init, Linear_Ops;
with Combined_Fou2_Ops;

generic

  Size_1, Size_2, Q_deg, P_deg, QPdeg: in Positive;
  type Scalar is private;


  with function IsNumeric(S: Scalar) return Boolean is <>;
  with function IsInterval(S: Scalar) return Boolean is <>;
  with procedure Get(F: in File_Type; S: out Scalar) is <>;
  with procedure Put(F: in File_Type; S: in Scalar) is <>;
  with procedure Show(S: in Scalar) is <>;
  with function Scal(K: Integer) return Scalar is <>;
  with function Scal(R: Rep) return Scalar is <>;
  with function Scal(E: Error) return Scalar is <>;
  with function Center0(S: Scalar) return Boolean is <>;
  with function Err0(S: Scalar) return Boolean is <>;
  with function Center(S: Scalar) return Rep is <>;
  with function ErrPart(S: Scalar) return Error is <>;
  with procedure ResetCenter(S: in out Scalar) is <>;
  with function ResetCenter(S: Scalar) return Scalar is <>;
  with procedure Split(S: in Scalar; Sr,Se: out Scalar) is <>;
  with function Inf(S: Scalar) return Rep is <>;
  with function Sup(S: Scalar) return Rep is <>;
  with function SupAbs(S: Scalar) return Rep is <>;
  with function Err(S: Scalar) return Error is <>;
  with procedure ResetErr(S: in out Scalar) is <>;
  with function ResetErr(S: Scalar) return Scalar is <>;
  with procedure ErrMult(Factor: in Error; S: in out Scalar) is <>;
  with function "abs"(S: Scalar) return Scalar is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with function "+"(Left,Right: Scalar) return Scalar is <>;
  with function "-"(Left,Right: Scalar) return Scalar is <>;
  with function "*"(S: Scalar; E: Error) return Error is <>;
  with function "*"(E: Error; S: Scalar) return Error is <>;
  with function "*"(Left,Right: Scalar) return Scalar is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "/"(E: Error; S: Scalar) return Error is <>;
  with function "/"(Left,Right: Scalar) return Scalar is <>;
  with function "**"(X1: Scalar; I: Integer) return Scalar is <>;
  with function Sqrt(S: Scalar) return Scalar is <>;
  with function ExpAbsLog(S: Scalar) return Scalar is <>;
  with function Cosh(S: Scalar) return Scalar is <>;
  with function Sinh(S: Scalar) return Scalar is <>;
  with procedure CoshSinh(S: in Scalar; Sc,Ss: out Scalar) is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with function Golden return Scalar is <>;


package RG2_Ops is

  package RG2_Fou2_Ops is
    new Combined_Fou2_Ops (Size_1 => Size_1, Size_2 => Size_2, Q_Deg => Q_Deg,
                           P_Deg  => P_Deg,   QPdeg => QPdeg, Scalar => Scalar);

  --  package Scalar_Taylor_Ops renames  RG2_Fou2_Ops.Scalar_Part.Tay1_Ops;
  package Numeric_Taylor_Ops  renames  RG2_Fou2_Ops.Numeric_Part.Tay1_Ops;


  use RG2_Fou2_Ops, Numeric_Taylor_Ops ;

  type DS_Matrix is array (Integer range 1..4,Integer range 1..4) of Scalar;
  type DS_Vector is array (Integer range 1..4) of Scalar;

  package Scalar_Fou2_Ops renames RG2_Fou2_Ops.Scalar_Part;
  package Scalar_Taylor_Ops renames  Scalar_Fou2_Ops.Tay1_Ops;
  package Scalar_Fou1_Ops renames RG2_Fou2_Ops.Scalar_Part.Fou1_Ops;  --@
  package Numeric_Fou2_Ops renames RG2_Fou2_Ops.Numeric_Part;
  package Numeric_Fou1_Ops renames RG2_Fou2_Ops.Numeric_Part.Fou1_Ops;  --@
  package BStrings is new Ada.Strings.Bounded.Generic_Bounded_Length(80);
  use BStrings;
  subtype Hamilton is Scalar_Fou2_Ops.Fourier2;
  subtype Polynom is Numeric_Fou2_Ops.Fourier2;
  subtype H_Freq1 is Scalar_Fou1_Ops.Freq1;
  subtype H_Freq2 is Scalar_Fou1_Ops.Freq2;

  type Svec2 is array(1 .. 2) of Scalar;
  type Smat2 is array(1 .. 2,1 .. 2) of Scalar;
  type Mu_Array is array(0..11) of Scalar;
  type Polynoms is array(0..11) of Polynom;
  type Hamiltons is array(0..11) of Hamilton;
  subtype BString is Bounded_String;


  type ProjParam is
     record
        Sigma: Rep;
        Kappa: Rep;
        V:     Rep;                           ---- not currently used in Iminus
        L:     Cutoffs(H_Freq1,H_Freq2);
        Count: Natural;
     end record;

  --  Cpp: constant Rep := Half;
  Cpp: constant Rep := Half;
  Theta: constant Scalar := Golden;
  MuMax: Rep ;
  EX: Integer:=0;
  Long: Integer:=0;
  Con: Rep;
  Exc: Rep;
  MaxI: Integer;
  Iterr: Integer;
  Rmu: Scalar; --:=Scal(One);
  Rvh0: Scalar;

  AuxSave: Boolean;
  AuxName: BString;



  function FixTrivial1(R: Weights) return Hamilton;                                                        --Returns omega*p+Cpp*(Omega*p)^3
  function FixTrivial2(R: Weights) return Hamilton;                                                        --Returns omega*p-Cpp*(Omega*p)^3
  procedure  Pack(H: in Polynom; P: in ProjParam; VH: out Vector);                                         -- Packs I^minus H into VH
  function CutPack(Least: Integer; Cut: Radius; Vf: Vector; P: ProjParam; H: Polynom) return Vector;       -- Returns packed H, cut using other args
  procedure UnPack(R: in Weights; P: in ProjParam; VH: in Vector; H: out Polynom);                         -- Unpacks V to H (type I^minus)
  procedure NormFactors(P: in ProjParam; R: in Weights; Vf: out Vector);                                   -- Makes norm factors
  procedure ShowProj(P: in ProjParam);                                                                     -- Show given ProjParam
  procedure ShowParam(Sigma,Kappa: in Rep);                                                                -- Does what is sais
  function IminusDeg(Sigma,Kappa: Rep; M,N: Integer; Check: in Boolean := True) return Integer;            -- Returns max degree for Iminus using the definition:
                                                                                                           -- Iminus:={|omega*nu|>Sigma*|Omega*nu|+Kappa*k}
  function IminusDegNew(Sigma,Kappa: Rep; M,N: Integer; Check: in Boolean := True) return Integer;         -- Returns max degree for Iminus using the definition:
                                                                                                           -- Iminus:={|omega*nu|>Sigma*|Omega*nu| and |omega*nu|>Kappa*k}
  function IminusCount(Sigma,Kappa: Rep) return Integer;                                                   -- Returns number of Iminus modes
  function IminusCountNew(Sigma,Kappa: Rep) return Integer;                                                -- Returns number of Iminus modes
  procedure IminusDef(Sigma,Kappa: in Rep; P: out ProjParam);                                              -- ProjParam for Iminus
  procedure IminusDefNew(Sigma,Kappa: in Rep; P: out ProjParam);                                           -- ProjParam for Iminus
  procedure VHatCheck(Sigma,Kappa,SigHat,KapHat,VHat1: in Rep; VHat2: out Rep);                            -- Decrease VHat, if necessary
  procedure Normalize(H: in Hamilton; Hn: out Hamilton);                                                   -- Hn := (H normalized)
  function  Ubound(R,Rp: Weights) return Scalar;                                                           -- Returns upper bound on U
  procedure SKCheck(R,Rp: in Weights; Sigma,Kappa,SigHat1,KapHat1: in Rep; SigHat2,KapHat2: out Rep);      -- Increase SigHat,KapHat, if possible
  procedure Shift(R: in Weights; PPi,PPj: ProjParam; H1: in Hamilton; H2: out Hamilton);                   -- Shift:  H2 := H1 o T_MuMax
  procedure HoU(Err: in Radius; P: in ProjParam; R: in Weights; H,Hpsi: in Hamilton; Hr: out Hamilton);    -- Hr := H o U_Hpsi
  procedure HoToU(Err: in Radius; P: in ProjParam; R: in Weights; H,Hpsi: in Hamilton; Hr: out Hamilton);  -- Hr := H o T o U_Hpsi
  procedure PsiOfUoU(Err: in Radius; P: in ProjParam; Psi1: in Polynom; Psi2: in out Polynom);             -- Solves U_Psi = U_Psi2 o U_Psi1, Psi2 := Psi
  function  Vguess(PPi: ProjParam; SigHat,KapHat: Rep) return Rep;                                         -- Guess for VHat
  procedure Hinc(T: in Scalar; H1: in Hamilton; H2: out Hamilton);                                         -- increment map for RGC, increment of component (0,0,0,1)
  procedure Hinc2(T: in Scalar; H1: in Hamilton; H2: out Hamilton);                                        -- increment map for RGC, increment of component (0,0,0,2)
  procedure Hinx(T: in Svec2; He1,He2,H1: in Hamilton; H2: out Hamilton);                                  -- increment map for RGX
  procedure Hinx(T: in Svec2; He1,He2: in Hamilton; H2: in out Hamilton);                                  -- increment map for RGX
  procedure Hinx(T: in Scalar; He1,H1: in Hamilton; H2: out Hamilton);                                     -- increment map for RGX, increment in direction of He1 only
  procedure Hinx(T: in Scalar; He1: in Hamilton; H2: in out Hamilton);                                     -- increment map for RGX, increment in direction of He1 only
  procedure Hincc(T: in Scalar; P: in ProjParam; H1: in Hamilton; H2: out Hamilton);                       -- increment map for RGCC2
  function  Nfunc(H1,H2: Hamilton) return Scalar;                                                          -- Normalization function for RGC
  function  Nfunc2(H1,H2: Hamilton) return Scalar;                                                         -- Normalization function for RGC, component (0,0,0,2)
  function  Nfunx(R: Weights; He1,He2,H1,H2: Hamilton) return Svec2;                                       -- Normalization function for RGX
  function  Nfunx(R: Weights; He1,H1,H2: Hamilton) return Scalar;                                          -- Normalization function for RGX, direction of He1 only
  function Nfuncc(H1,H2: Hamilton) return Scalar;                                                          -- Normalization function for RGCC2
  procedure StarStuff(Err: in Radius; PPi: in ProjParam; Mu: in Scalar;Hpsi: in Polynom;
                                                                       Zstar,Lambda2: out Scalar);         -- Finds scaling point and eigenvalue
  procedure ShowRR(R: in Weights);                                                                         -- Shows the weights
  function RoughPsi(P: ProjParam; H: Hamilton; R: Weights) return Polynom;                                 -- rough psi (not necessarily I-)
  function RoughPsiNoShift(P: ProjParam; H: Hamilton; R: Weights) return Polynom;                          -- rough psi (not necessarily I-), not preshifted
  procedure DS(Err: in Radius; PPi: in ProjParam; Rt: in Weights;Hpsi: in Polynom; Q1,Q2,Z: in Scalar;
                                                                                   DS: out DS_Matrix);     -- derivative of the scaling transfromation at (Q1,Q2,Z)
  procedure DS(Err: in Radius; PPi: in ProjParam; Rt: in Weights; DSIname,MuIname: in String;
                                                                  Q1,Q2,Z: in Scalar; DS: out DS_Matrix);  -- DS which reads data from files DSIname and MuIname
  procedure Mult(A1: in out DS_Matrix; A2: in  DS_Matrix);                                                 -- A1:=A1*A2
  procedure ShowDS(A: in DS_Matrix);                                                                       -- Shows matrix DS
  procedure FindEV(Iname,Oname: in String;Pw,P: in Integer; Err: in Radius; PPi: in ProjParam;
                                                                            Rf,Rpsi: in Weights;
                                                                            H: in Hamilton;
                                                                            H1,H2: in out Hamilton;
                                                                            Read_Psi: in Boolean;
                                                                            Text_Output: in Boolean:=True);-- Finds ren eigenvectors
  function Components(R: Weights; H1,H2,H: Hamilton) return Svec2;                                         -- Components of H along H1,H2
  procedure Rotate_Scale(Rt: in Weights; S1, S2, S3: in Scalar; H1: in Hamilton; H2: out Hamilton);        -- H2(x,y,z):=H1(x+S1,y+S2,S3*z)
  procedure MultAdd(W: in Sparse; H1: in Hamilton; H2: in out Hamilton);                                   -- H2 := H2+W*H1 (assuming MNK indexing)
  procedure ReadDRG(Rfix: in Weights; Iname: in String; DR: out Matrix; Iz,Jz: out Pivot;                  -- DR:=(DRG-Id)^{-1}; attempts to read columns from directory
                                                                        Text_Output: in Boolean:=True);    -- Iname; if Text_Output then log in the current output
  procedure NumDRG(Iname,Oname: in String; Pw,P: in Integer; Mmin,Mmax: in Integer;
                                                             Err,CutOff: in Rep;
                                                             PPi: in ProjParam;
                                                             Rf,Rpsi: in Weights;
                                                             H1,H2: in Hamilton;                           -- Derivative of RG; if Read_Psi then read Psi's from file Iname;
                                                             Read_Psi: in Boolean;                         -- range of M's for DRG is given by Mmin and Mmax;
                                                             Text_Output: in Boolean:=True);               -- if Text_Output then log in the current output
  function Join(I,M,N,K: Integer) return Integer;                                                          -- (M,N,K) -> ((100+M)*100+N)*100+K
  procedure UnJoin(MNK: in Integer; I,M,N,K: out Integer);                                                 -- Does what it sais
  procedure NameForm(P,M,N,K: in Integer; II,MM,NN,KK: out String);                                        -- Forms names for files containing columns of DRG
  procedure UnjoinUnpack(V: in Vector; Rfix: in Weights; H: out Hamilton);                                 -- Does what it sais
  procedure JoinPack(H: in Hamilton; V: out Vector);                                                       -- Does what it sais
  procedure MultAdd(W: in Matrix; Iz,Jz: in Pivot; H1: in Hamilton; H2: in out Hamilton);
  procedure ST(Switch: in Integer; ScalName,PsiName,MuName: in String; Err: in Radius;
                                                                       P: in ProjParam;
                                                                       R: in Weights;                      -- Value of the scaling transformation at (Q1,Q2,Z),
                                                                       Q1,Q2,Z: in out Scalar);            -- uses Psi's and Mu's from directories PsiName and MuName
  procedure K_Term(K: in Integer; Err: in Radius; P: in ProjParam; R: in Weights; Hpsi: in Hamiltons;
                                                                                  Mu: in Mu_Array;
                                                                                  Y,Z: out Hamilton);      -- Used in ST

  ----------- Polynom computations
  procedure HoUnum(Err: in Radius; P: in ProjParam; H,Hpsi: in Polynom; Hr: out Polynom);                  -- Hr := H o U_Hpsi
  procedure THoUmat2(Err: in Radius; P: in ProjParam; Vf: in Vector; H,Hpsi: in Polynom;
                                                                     Hr: out Polynom;
                                                                     W: out Matrix);                       -- Hr := H o U_Hpsi, W=d(Hr)/d(Hpsi)
  procedure THoToUmat2(Err: in Radius; P: in ProjParam; Vf: in Vector; H,Hpsi: in Polynom; Hr: out Polynom;
                                                                                           W: out Matrix); -- Hr := H o T o U_Hpsi, W=d(Hr)/d(Hpsi)
  procedure THoUmat3(Err: in Radius; P: in ProjParam; Vf: in Vector; H,Hpsi: in Polynom; Hr: out Polynom;
                                                                                         W: out Sparse);   -- sparse matrix version of THoUmat2
  procedure PsiFind_Sparse(Err: in Radius; R: in Weights; P: in ProjParam; H: in Hamilton;
                                                                           Hpsi: in out Polynom;
                                                                           W: in out Sparse);              -- sparse matrix version of PsiFind
  procedure PsiFind(Err: in Radius; R: in Weights; P: in ProjParam; H: in Hamilton; Hpsi: in out Polynom); -- Solve I^minus (H o U_Hpsi) = 0
  procedure PsiFind2(Err: in Radius; R: in Weights; P: in ProjParam; H: in Hamilton; Hpsi: in out Polynom);-- Solve I^minus (H o T o U_Hpsi) = 0

  -- RENORMALIZATION PROCEDURES

  procedure RG1(Err: in Radius; PPi: in ProjParam; Rt,Rf: in Weights; H1: in Hamilton; Hpsi: in out Polynom;
                                                                                       W: in out Sparse;
                                                                                       H2: out Hamilton);   -- Plain RG, uses PsiFind_Sparse, HoUoT
  procedure RG2(Err: in Radius; PPi: in ProjParam; Rt,Rf: in Weights; H1: in Hamilton; Hpsi: in out Polynom;
                                                                                       H2: out Hamilton;
                                                                                       N: in Integer:=0);   -- Plain RG, uses PsiFind, HoToU
  procedure RG3(Err: in Radius; PPi: in ProjParam; Rt,Rf: in Weights; H1: in Hamilton; Hpsi: in out Polynom;
                                                                                       H2: out Hamilton;
                                                                                       N: in Integer:=0);   -- Plain RG, does HoToU first, then adjusts
  procedure RG_Pw(Iname,Oname: in String; Pw,P: in Integer; Err: in Radius; PPi: in ProjParam;
                                                                            Rf,Rpsi: in Weights;            -- RG^Pw o (Q-Translation)
                                                                            H1: in Hamilton;                --starting from a Hamiltonian with the simmetry of P-th
                                                                            H2: out Hamilton;               -- point in the period; if Read_Psi/Write_Psi then read/write
                                                                            Read_Psi,Write_Psi:in Boolean );-- Psi's from/to Iname/Oname
  procedure RGC(Iname,Oname: in String; Pw,P: in Integer; Eps: in Rep; Err: in Radius;
                                                                       PPi: in ProjParam;
                                                                       Rf,Rpsi: in Weights;
                                                                       H1: in Hamilton;
                                                                       H2: out Hamilton;
                                                                       Read_Psi,Write_Psi:in Boolean );     -- RG_Pw with contraction in the trivial unstable direction
  procedure RGX(Iname,Oname: in String;Pw,P: in Integer; Err: in Radius; PPi: in ProjParam;
                                                                         Rf,Rpsi: in Weights;
                                                                         He1,He2: in out Hamilton;
                                                                         H1: in out Hamilton;
                                                                         H2: out Hamilton;
                                                                         Read_Psi,Write_Psi: in Boolean);   -- RG_Pw with contraction in the directions of He1 and He2
  procedure RGY(Iname,Oname: in String; Pw,P: in Integer; Err: in Radius; PPi: in ProjParam;
                                                                          Rf,Rpsi: in Weights;
                                                                          H1: in out Hamilton;
                                                                          Y: in Matrix;
                                                                          Iz,Jz: in Pivot;
                                                                          H2: out Hamilton;
                                                                          Read_Psi,Write_Psi: in Boolean);  -- RG_Pw with contraction; uses  RG derivarive
end RG2_Ops;








